package CalculiX;

class EDoubleComparison extends Expression {

  // The comparator.
  final Comparator op;
  // The operands.
  final Expression left, right;

  EDoubleComparison (Expression left, Comparator op, Expression right)
  {
    this.left = left;
    this.op = op;
    this.right = right;
  }

  public Value eval (Environment env) throws EvalException
  {
    // Evaluate the operands.
    Value v1 = left.eval(env);
    Value v2 = right.eval(env);
    // Make sure that they produce double results.
    double d1 = v1.asDouble();
    double d2 = v2.asDouble();
    // Perform the appropriate operation.
    boolean b;
    switch (op) {
    case LESSTHAN:
      b = d1 < d2;
      break;
    case LESSTHANOREQUALTO:
      b = d1 <= d2;
      break;
    case EQUALS:
	b = (d1 == d2);
      break;
    default:
      // This cannot happen.
      throw new AssertionError ();
    }
    // Package up the result as a value.
    return new VBoolean (b);
    }

}
