package CalculiX;

public class Util {

  // The parser is generated by JavaCUP, an LR(1) parser generator.
  // The grammar is in Parser.cup.

  // An instance of factory.
  private static ExpressionFactory factory = new MyExpressionFactory();

  // The initial environment.
  public static final Environment INITIAL = Environment.empty.extend("pi",
      new VDouble(Math.PI)).extend("e", new VDouble(Math.E));

  // This method initializes and runs the parser.
  // It requires a Reader and an expression factory.
  // It returns an expression.
  static Expression parse(java.io.Reader reader, ExpressionFactory factory)
      throws Exception {
    // Set up a lexer.
    CalculiXLexer lexer = new CalculiXLexer(reader);
    // Set up the parser.
    MyParser parser = new MyParser(lexer);
    // Allow the parser to access the expression factory.
    CUP$parser$actions.factory = factory;
    // Run the parser.
    // The method parse() produces a Symbol, whose semantic value is
    // an abstract syntax tree for an expression.
    return (Expression) parser.parse().value;
  }

  public static Expression parse(String text) throws Exception {
    // Read from the supplied string.
    java.io.Reader reader = new java.io.StringReader(text);
    // Parse.
    return parse(reader, factory);
  }

  // This method runs the interpreter.
  public static double eval(Expression e, Environment env) throws Exception {
    return e.eval(env).asDouble();
  }

  // This method adds a pair (name,value) to the given environment and returns a
  // new resulting environment
  public static Environment extend(Environment env, String name, double value) {
    return env.extend(name, new VDouble(value));
  }
}
