package CalculiX;

// This interface describes an object that is able to build
// expressions of each kind. We refer to such an object as an
// "expression factory".

// When constructing abstract syntax trees, the parser relies on an
// expression factory instead of creating objects directly via "new".
// This allows the parser to be self-contained (that is, even if the
// various sub-classes of Expression have not been defined yet, the
// parser can be compiled). This is exploited by TD Interpretation.

interface ExpressionFactory {

  Expression buildConstant (double d);

  Expression buildConstant (boolean b);

  Expression buildDoubleOperation (Expression left, Operator op, Expression right);

  Expression buildBooleanOperation (Expression left, Connective op, Expression right);

  Expression buildBooleanNegation (Expression e);

  Expression buildDoubleComparison (Expression left, Comparator op, Expression right);

  Expression buildIf (Expression condition, Expression then_branch, Expression else_branch);

  Expression buildLet (String x, Expression left, Expression right);

  Expression buildVarRead (String x);

  Expression buildFunApp (String f, Expression e);

}
